import { Plugin, PluginServiceApi } from '@cloudbase/framework-core';
import * as CI from 'miniprogram-ci';
/**
 * 导出接口用于生成 JSON Schema 来进行智能提示
 */
interface IFrameworkPluginMiniProgramInputs {
    /**
     * 小程序应用的 appid
     */
    appid: string;
    /**
     * 小程序应用的部署私钥的本地相对路径
     *
     * @default "./private.key"
     */
    privateKeyPath: string;
    /**
     *
     * 小程序应用的部署私钥（需要经过base64编码）
     */
    privateKey?: string;
    /**
     * 小程序项目的本地相对路径
     *
     * @default "./"
     */
    localPath: string;
    /**
     * 小程序应用部署时忽略的文件路径，支持通配符
     *
     * @default ["node_modules\/**\/*"]
     */
    ignores?: string[];
    /**
     * 小程序应用的部署模式
     *
     * @default "preview"
     */
    deployMode?: 'preview' | 'upload';
    /**
     * 预览代码的选项
     */
    previewOptions?: IMiniProgramPreviewOptions;
    /**
     * 上传代码的选项
     */
    uploadOptions?: IMiniProgramUploadOptions;
    /**
     * 自定义命令
     *
     * @default { build: "cd ./miniprogram && npm install" }
     */
    commands?: Record<string, string>;
}
interface IMiniProgramUploadOptions {
    /**
     * 小程序应用上传的版本号
     */
    version?: string;
    /**
     * 小程序应用的版本描述
     *
     * @default "CloudBase Framework 一键上传"
     */
    desc?: string;
    /**
     * 小程序应用的编译设置
     */
    setting?: IMiniProgramBuildSetting;
}
interface IMiniProgramPreviewOptions {
    /**
     * 小程序的版本描述
     *
     * @default "CloudBase Framework 一键预览"
     */
    desc?: string;
    /**
     * 小程序应用的编译设置
     */
    setting?: IMiniProgramBuildSetting;
    /**
     * 生成的预览二维码保存在本地的路径
     */
    qrcodeOutputPath?: string;
    /**
     * 小程序应用的预览页面地址
     *
     * @default pages/index/index
     */
    pagePath?: string;
    /**
     * 小程序应用的预览页面参数
     *
     * @default
     */
    searchQuery?: string;
    /**
     * 小程序应用的预览页面场景值
     *
     * @default 1011
     */
    scene?: number;
}
interface IMiniProgramBuildSetting {
    /**
     * 编译设置 - es6 转 es5
     *
     * @default true
     */
    es6?: boolean;
    /**
     * 编译设置 - 压缩代码
     *
     * @default true
     */
    minify?: boolean;
    /**
     * 编译设置 - 开启代码保护
     *
     * @default true
     */
    codeProtect?: boolean;
}
interface IMiniProgramBuildOutput {
    /**
     * 预览版二维码链接
     */
    link?: string;
    /**
     * 体验版小程序版本号
     */
    version?: string;
}
declare class MiniProgramsPlugin extends Plugin {
    name: string;
    api: PluginServiceApi;
    inputs: IFrameworkPluginMiniProgramInputs;
    protected resolvedInputs: IFrameworkPluginMiniProgramInputs;
    protected buildOutput: IMiniProgramBuildOutput;
    constructor(name: string, api: PluginServiceApi, inputs: IFrameworkPluginMiniProgramInputs);
    /**
     * 初始化
     */
    init(): Promise<void>;
    _initCI(): void;
    /**
     * 执行本地命令
     */
    run(): Promise<void>;
    /**
     * 删除资源
     */
    remove(): Promise<void>;
    /**
     * 生成代码
     */
    genCode(): Promise<void>;
    /**
     * 构建
     */
    build(): Promise<void>;
    /**
     * 生成SAM文件
     */
    compile(): Promise<{}>;
    /**
     * 部署
     */
    deploy(): Promise<void>;
    /**
     * 小程序-上传
     */
    _ciUpload(): Promise<IMiniProgramBuildOutput>;
    /**
     * 小程序-预览
     */
    _ciPreview(): Promise<IMiniProgramBuildOutput>;
    /**
     * 小程序-编译NPM
     */
    _ciPackNpm(): Promise<void>;
    get _ciProject(): CI.Project;
}
export declare const plugin: typeof MiniProgramsPlugin;
export {};
