import fs from 'fs-extra';
import archiver from 'archiver';
import { Project as MpProject, packNpm, preview as previewMp, upload as uploadMp, analyseCode } from 'miniprogram-ci';
import QRCode from 'qrcode';
import path from 'path';
import { CloudApiService } from '@cloudbase/cloud-api';
import { TCECloudApiService } from './utils/tce-cloud-api/cloud_api_service';
import CloudBase from '@cloudbase/manager-node';
import { AsyncTaskParallelController } from '@cloudbase/manager-node/lib/utils/parallel';
import { PRIVATE_JS_CODE, PRIVATE_JS_NAME } from './config/private';
import { StaticBuilder } from '@cloudbase/static-builder';
import { emoji } from './utils/emoji';
import { BuildType, buildAsAdminPortalByBuildType, buildAsXPageByBuildType } from '@cloudbase/lowcode-builder';
import { Storage, getUploadDirFileList } from './utils/storage';
import { ICredentialOptions } from './types';
import url from 'url';

const NOT_NPM_ERROR = '__NO_NODE_MODULES__ NPM packages not found';
const QRCODE_PATH = './qrcode.jpg';

interface IDeployerOptions {
  api?: { emoji: (str: string) => string };
  cloudbaseManager?: any;
  logger?: any;
}
class Deployer {
  protected logger;
  protected api: { emoji: (str: string) => string };
  constructor(options: IDeployerOptions) {
    // this.app = options.cloudbaseManager;
    // this.logger = getLogger();
    this.logger = options.logger || console;
    this.api = options.api || { emoji };
  }
}

export enum EDeployMode {
  PRODUCTION = 'PRODUCTION',
  PREVIEW = 'PREVIEW',
}

interface ILowcodeDeployerOptions extends IDeployerOptions {
  type: 'WEB' | 'MP';
  mode: EDeployMode;
  appId: string;
  envId: string;
  projectPath: string;
  outputPath?: string;
  // web 部署密钥
  credential?: ICredentialOptions;
  // ignore path list
  ignores?: string[];
  mpAppId?: string;
  miniprogramOptions?: {
    ciProject?: any;
    privateKeyPath?: string;
    privateKey?: string;
    projectConfig?: any;
    uploadOptions?: any;
    previewOptions?: IMiniProgramPreviewOptions;
  };
  webOptions?: {
    cloudPath?: string;
    buildTypeList?: BuildType[];
    gatewayConfig?: {
      domain: string;
      path: string;
    };
    /**
     * 私有化使用自定义上传endpoint
     */
    endpoint?: string;
    /**
     * 自定义配置上传存储桶
     */
    bucket?: string;
    region?: string;
    credential?: ICredentialOptions;
  };
  artifactOptions?: {
    bucket: string;
    baseKey: string;
    region?: string;
    credential?: ICredentialOptions;
    endpoint?: string;
  };
}

interface IMiniProgramPreviewOptions {
  /**
   * 小程序的版本描述
   *
   * @default "CloudBase Framework 一键预览"
   */
  desc?: string;
  /**
   * 小程序应用的编译设置
   */
  setting?: ICompileSettings;
  /**
   * 生成的预览二维码保存在本地的路径
   */
  qrcodeOutputPath?: string;
  /**
   * 小程序应用的预览页面地址
   *
   * @default pages/index/index
   */
  pagePath?: string;
  /**
   * 小程序应用的预览页面参数
   *
   * @default
   */
  searchQuery?: string;
  /**
   * 小程序应用的预览页面场景值
   *
   * @default 1011
   */
  scene?: number;
}

interface ICompileSettings {
  es6?: boolean;
  es7?: boolean;
  minify?: boolean;
  codeProtect?: boolean;
  minifyJS?: boolean;
  minifyWXML?: boolean;
  minifyWXSS?: boolean;
  autoPrefixWXSS?: boolean;
  disableUseStrict?: boolean;
}

interface IMiniProgramBuildOutput {
  /**
   * 预览版二维码链接
   */
  link?: string;
  /**
   * 体验版小程序版本号
   */
  version?: string;
}

export default class LowcodeDeployer extends Deployer {
  protected credential;
  public mpCIProject;
  protected resolvedInputs: Required<ILowcodeDeployerOptions>;
  protected lowcodeService;
  protected tcbService;
  protected cloudbaseManager;
  constructor(options: ILowcodeDeployerOptions) {
    super(options);
    this.credential = options.credential;
    this.resolvedInputs = this._resolveInputs(options);

    const { type, mpAppId, projectPath, ignores, miniprogramOptions } = this.resolvedInputs;

    if (type === 'MP') {
      this.mpCIProject =
        miniprogramOptions?.ciProject ||
        this._generateMpCIProject({
          mpAppId,
          projectPath,
          privateKeyPath: miniprogramOptions?.privateKeyPath,
          ignores: ['node_modules/**/*', ...ignores],
          ...miniprogramOptions?.projectConfig,
        });
    } else {
      this.lowcodeService = this._generateLowcodeService(this.credential);
      this.tcbService = this._generateTcbService(this.credential);
      this.cloudbaseManager = new CloudBase({
        ...this.credential,
        envId: this.resolvedInputs.envId,
      });
    }
  }

  private _resolveInputs(options: ILowcodeDeployerOptions) {
    const resolvedInputs = {
      ...options,
      outputPath: options.outputPath || options.projectPath,
      ignores: options.ignores || [],
    };

    if (resolvedInputs.type === 'MP') {
      const { miniprogramOptions = {}, projectPath } = resolvedInputs;
      let { privateKey, privateKeyPath = '' } = miniprogramOptions;

      if (privateKey) {
        const keyPath = path.resolve(projectPath, './private.key');
        const key = Buffer.from(privateKey, 'base64');
        fs.writeFileSync(keyPath, key.toString());
        privateKeyPath = keyPath;
      }

      if (!privateKeyPath || !fs.existsSync(path.resolve(projectPath, privateKeyPath))) {
        throw new Error('need privateKey or privateKey in miniprogramOptions config');
      }
      miniprogramOptions.privateKeyPath = privateKeyPath;
      resolvedInputs.miniprogramOptions = miniprogramOptions;
    } else if (resolvedInputs.type === 'WEB') {
      let { webOptions = {}, credential } = resolvedInputs;
      if (!credential || (process.env.IS_PRIVATE && !webOptions.credential)) {
        throw new Error('need credential to deoploy website');
      }
      (webOptions as any).cloudPath = this._getWebRootPath(resolvedInputs);
      if (!webOptions.buildTypeList) {
        webOptions.buildTypeList = [];
      }
      resolvedInputs.webOptions = webOptions;
    }

    if (resolvedInputs.artifactOptions) {
      if (!resolvedInputs.artifactOptions.baseKey) {
        throw new Error('缺少制品持久化目标地址配置');
      }
      if (!resolvedInputs.artifactOptions.bucket) {
        throw new Error('缺少制品持久存储桶配置');
      }
      resolvedInputs.artifactOptions = {
        ...resolvedInputs.artifactOptions,
        credential:
          resolvedInputs.artifactOptions.credential ||
          (process.env.IS_PRIVATE ? resolvedInputs.webOptions?.credential : this.credential),
        endpoint: resolvedInputs.artifactOptions.endpoint || resolvedInputs.webOptions?.endpoint,
        region: resolvedInputs.artifactOptions.region || resolvedInputs.webOptions?.region,
      };
    }

    return resolvedInputs as Required<typeof resolvedInputs>;
  }

  private _getWebRootPath(inputs) {
    let { appId, mode } = inputs;
    return mode === EDeployMode.PREVIEW ? `/${appId}/preview/` : `/${appId}/production/`;
  }

  async deploy(options = {}) {
    const { type, mode, credential } = this.resolvedInputs;
    if (type === 'MP') {
      try {
        await this._genearteMiniprogramNPM();
        const list = await this._cleanNonRefFile();
        this.logger?.info?.(list.join('/n'));
      } catch (e) {
        this.logger?.debug?.((e as any).message);
      }
      return mode === EDeployMode.PRODUCTION ? this._ciUpload(options) : this._ciPreview(options);
    } else if (type === 'WEB') {
      const { webOptions } = this.resolvedInputs;
      try {
        await this.lowcodeService.request('UpdateLoginConfig', { EnvId: this.resolvedInputs.envId });
        this.logger?.debug('login config set success');
      } catch (e) {
        this.logger.error(`设置登录配置失败 ${(e as any).message}}`);
      }

      const { website, domains = [] } = await this._preProcessWebDeploy();

      if (!buildAsXPageByBuildType(webOptions.buildTypeList)) {
        const ignores = ['.tmp', '.git', '.github', 'node_modules', 'cloudbaserc.js', ...this.resolvedInputs.ignores];
        const includes = ['**', ...ignores.map((ignore) => `!${ignore}`)];

        const builder = new StaticBuilder({
          projectPath: path.resolve(this.resolvedInputs.projectPath),
        });

        const { static: staticList = [], staticConfig = [] } = await builder.build(includes, {
          path: this.resolvedInputs.webOptions.cloudPath,
          domain: process.env.IS_PRIVATE ? undefined : website?.CdnDomain,
        });
        const deployContent = [...staticList];

        if (process.env.IS_PRIVATE) {
          const { endpoint, region, bucket, credential } = webOptions as any;
          const instance = new Storage({ endpoint, credential, region, cos: false });
          const fileList = deployContent.reduce(
            (list, meta) => {
              let cloudPath = meta.cloudPath;
              if (cloudPath.startsWith('/')) {
                cloudPath = cloudPath.substring(1);
              }
              list.push(...getUploadDirFileList(meta.src, cloudPath));
              return list;
            },
            [] as {
              src: string;
              cloudPath: string;
            }[],
          );

          const asyncTaskController = new AsyncTaskParallelController(20, 50);
          asyncTaskController.loadTasks(
            fileList.map((meta) => {
              return async () => {
                try {
                  const param = {
                    Bucket: bucket as string,
                    ACL: 'public-read',
                    Body: fs.createReadStream(meta.src),
                    Key: meta.cloudPath,
                  };
                  try {
                    await instance.putObject(param);
                  } catch (e) {
                    // 间隔重试一次
                    await new Promise((resolve, reject) => {
                      setTimeout(() => {
                        resolve(null);
                      }, 500);
                    });
                    await instance.putObject(param);
                  }
                  return { code: 0 };
                } catch (e: any) {
                  if (!e.code) {
                    e.code = -1;
                  }
                  throw e;
                }
              };
            }),
          );
          const list = await asyncTaskController.run();
          if (list.some((item) => item.code)) {
            console.log(list.filter((item) => item.code));
            throw new Error('部分文件部署失败');
          }
        } else {
          /**
           * 私有化场景该文件交给初始化生成，不再兜底
           * 只有公有场景下防止环境未初始化情形，进行一次兜底
           */
          // 写入私有化JS文件占位
          const privateJsPath = path.join(this.resolvedInputs.projectPath, '.tmp', PRIVATE_JS_NAME);
          try {
            await fs.ensureFile(privateJsPath);
            await fs.writeFile(privateJsPath, PRIVATE_JS_CODE, {
              encoding: 'utf8',
              flag: 'w',
            });
          } catch (e) {
            this.logger.error('创建私有化JS文件失败:', e);
            throw e;
          }
          deployContent.push(...staticConfig);
          deployContent.push({ src: privateJsPath, cloudPath: `/weda-config/${PRIVATE_JS_NAME}` });
          await Promise.all(
            deployContent.map((options) =>
              this.cloudbaseManager.hosting.uploadFiles({
                localPath: options.src,
                cloudPath: options.cloudPath,
                ignore: ignores,
              }),
            ),
          );
          try {
            await fs.remove(privateJsPath);
          } catch (e) {}
        }

        try {
          await builder.clean();
        } catch (e) {}
      }

      await this._postProcessWebDeploy({ domains });
    }

    await this._persistArtifactOptions();
  }

  async _persistArtifactOptions() {
    if (this.resolvedInputs.artifactOptions) {
      const { endpoint, bucket, region, baseKey, credential } = this.resolvedInputs.artifactOptions;
      try {
        fs.ensureDirSync(path.resolve('.tmp'));
        const zipPath = path.resolve('.tmp', `${this.resolvedInputs.appId}.zip`);
        await zipDir(path.resolve(this.resolvedInputs.projectPath), zipPath);
        const storage = new Storage({
          endpoint,
          region,
          credential: credential as ICredentialOptions,
          cos: process.env.IS_PRIVATE ? false : true,
        });

        await storage.putObject({
          ACL: 'public-read',
          Bucket: bucket || '',
          Key: path.join(baseKey, 'dist.zip'),
          Body: fs.createReadStream(zipPath),
        });
        fs.removeSync(zipPath);

        if (fs.existsSync(path.resolve(this.resolvedInputs.outputPath, QRCODE_PATH))) {
          await storage.putObject({
            ACL: 'public-read',
            Bucket: bucket || '',
            Key: path.join(baseKey, 'qrcode.jpg'),
            Body: fs.createReadStream(path.resolve(this.resolvedInputs.outputPath, QRCODE_PATH)),
          });
        }

        this.logger.info(`${this.api.emoji('🚀')} 持久化制品成功。`);
      } catch (e) {
        this.logger.error(`${this.api.emoji('🚀')} 持久化上传制品失败：`, e);
      }
    }
  }

  async _preProcessWebDeploy() {
    const { envId } = this.resolvedInputs;

    if (process.env.IS_PRIVATE) {
      return { domains: [`${envId}.weda.xxxxx.cn`], website: { CdnDomain: `${envId}.weda.xxxxx.cn` } };
    }
    let domains, currentWebsite;
    try {
      let [website, hostingDatas] = await this._getHostingInfoTask(envId, false);
      if (!website) {
        throw new Error('静态托管初始化超时');
      }
      domains = hostingDatas.map((item) => item.CdnDomain);
      currentWebsite = website;
    } catch (e) {
      this.logger.error('获取静态托管失败: ', e);
      throw e;
    }
    return { domains, website: currentWebsite };
  }

  async _postProcessWebDeploy({ domains = [] }) {
    const { mode, appId, envId, webOptions, outputPath } = this.resolvedInputs;

    if (buildAsAdminPortalByBuildType(webOptions.buildTypeList)) {
      await this._postProcessAdminPortal();
    } else {
      await this._postProcessWebsiteConfig(domains);
    }

    const defaultCdnDomain = domains[0];
    try {
      let publishAppCustomNavRes = await this.lowcodeService.request('PublishAppCustomNav', {
        WeAppId: appId,
        EnvId: envId,
        PublishType: mode === EDeployMode.PRODUCTION ? 'product' : 'preview',
      });

      this.logger.info(`custom publish success: ${JSON.stringify(publishAppCustomNavRes)}`);
    } catch (e) {
      this.logger.error(`custom publish fail: ${JSON.stringify(e)}`);
      if ((e as any)?.code !== 'ResourceNotFound' && !/app does not exist/.test((e as any)?.message)) {
        throw e;
      }
    }

    try {
      const isPreview = mode === EDeployMode.PREVIEW;

      const link = buildAsAdminPortalByBuildType(webOptions.buildTypeList)
        ? `https://${defaultCdnDomain}/adminportal/#/app/${isPreview ? `${appId}-preview` : appId}?envType=${
            isPreview ? 'preview' : 'prod'
          }`
        : webOptions.gatewayConfig
        ? `https://${webOptions.gatewayConfig.domain + webOptions.gatewayConfig.path}`
        : `https://${defaultCdnDomain + webOptions.cloudPath}`;

      const qrcodeOutputPath = path.resolve(outputPath, QRCODE_PATH);
      await QRCode.toFile(qrcodeOutputPath, link, {
        errorCorrectionLevel: 'M',
        type: 'image/jpeg',
        scale: 12,
        margin: 2,
      });
      this.logger.info(`${this.api.emoji('🚀')} 网站部署成功：${link}`);
      this.logger.info(
        `${this.api.emoji('🚀')} 网站部署成功, 访问二维码地址：${url.format({
          protocol: 'file:',
          host: qrcodeOutputPath,
        })}`,
      );
    } catch (e) {
      this.logger.error('网站部署失败: ', e);
      throw e;
    }
  }

  async _postProcessAdminPortal() {
    if (process.env.IS_PRIVATE) {
      return;
    }
    try {
      if (this.resolvedInputs.mode === EDeployMode.PRODUCTION) {
        await this.lowcodeService.request('PublishAppCustomUrl', {
          EnvId: this.resolvedInputs.envId,
          WeAppId: this.resolvedInputs.appId,
        });
      }
      this.logger.debug('PublishAppCustomUrl成功');
    } catch (e) {
      this.logger.error('PublishAppCustomUrl失败: ', e);
    }
  }

  async _postProcessWebsiteConfig(domains) {
    if (process.env.IS_PRIVATE) {
      return;
    }
    const { mode, appId, envId, webOptions } = this.resolvedInputs;
    const hostingService = this.cloudbaseManager.hosting;
    try {
      let { Domains: domainList } = await hostingService.tcbCheckResource({
        domains,
      });
      // hostingService.getInfo();
      await Promise.all(
        domainList
          .filter((item) => item.DomainConfig.FollowRedirect !== 'on')
          .map((item) =>
            hostingService.tcbModifyAttribute({
              domain: item.Domain,
              domainId: item.DomainId,
              domainConfig: { FollowRedirect: 'on' } as any,
            }),
          ),
      );
      await Promise.all([
        mode === EDeployMode.PRODUCTION
          ? this.lowcodeService
              .request('PublishAppCustomUrl', {
                EnvId: envId,
                WeAppId: appId,
                IndexPath: webOptions.cloudPath,
              })
              .catch((e) => {
                this.logger.error('自定义域名发布失败: ', e);
                if (e?.code !== 'ResourceNotFound') {
                  throw e;
                }
              })
          : Promise.resolve(),
      ]);
    } catch (e) {
      this.logger.error('网站路由注册失败: ', e);
      throw e;
    }
  }

  private async _getHostingInfoTask(envId, loose = false) {
    let timeout: any = null;
    let [website, hostingDatas] = await Promise.race([
      new Promise((resolve) => {
        timeout = setTimeout(() => {
          resolve([]);
        }, 300 * 1000);
      }),
      this._getHostingInfo(envId, loose),
    ]);
    if (timeout) {
      clearTimeout(timeout);
    }
    return [website, hostingDatas];
  }

  private async _getHostingInfo(envId, loose = false) {
    let [website, hostingDatas] = await this.tcbService
      .request('DescribeStaticStore', { EnvId: envId })
      .then(({ Data: hostingDatas }) => {
        let website = hostingDatas[0];
        return [website, hostingDatas];
      });
    // 此处采取松校验
    if (!website || (loose ? false : website?.Status !== 'online')) {
      await new Promise((resolve) => {
        setTimeout(() => {
          resolve(true);
        }, 8 * 1000);
      });
      return this._getHostingInfo(envId, loose);
    } else {
      return [website, hostingDatas];
    }
  }

  private _generateLowcodeService(credential) {
    const params = {
      service: 'lowcode',
      version: '2021-01-08',
      timeout: 10000,
      credential,
    };
    return process.env.IS_PRIVATE
      ? new TCECloudApiService({
          ...params,
          baseUrl: process.env.PRIVATE_CLOUD_API_DOMAIN || 'yunapi3.weda-private.tcs230.fsphere.cn',
        })
      : new CloudApiService(params);
  }

  private _generateTcbService(credential) {
    const params = {
      service: 'tcb',
      version: '2018-06-08',
      credential,
    };
    return process.env.IS_PRIVATE
      ? new TCECloudApiService({
          ...params,
          baseUrl: process.env.PRIVATE_CLOUD_API_DOMAIN || 'yunapi3.weda-private.tcs230.fsphere.cn',
        })
      : new CloudApiService(params);
  }

  private _generateMpCIProject({ mpAppId: appid, projectPath, privateKeyPath, ignores }) {
    return new MpProject({
      appid,
      type: 'miniProgram',
      projectPath,
      privateKeyPath,
      ignores,
    });
  }

  /**
   * 小程序-编译NPM
   */
  private async _genearteMiniprogramNPM() {
    // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
    // this.api.runtime.isLocal() && this.api.console.pause();
    !process.env.CLOUDBASE_CIID && pauseConsoleOutput();
    const result = await packNpm(this.mpCIProject, {
      reporter: (infos) => {
        console.log(infos);
      },
    }).catch((err) => {
      return err;
    });
    // this.api.runtime.isLocal() && this.api.console.resume();
    !process.env.CLOUDBASE_CIID && resumeConsoleOutput();

    if (result instanceof Error && !result.message.startsWith(NOT_NPM_ERROR)) {
      throw new Error(`小程序 NPM 构建失败 ${result}`);
    }
  }

  private async _cleanNonRefFile() {
    const analyseRes = await analyseCode(this.mpCIProject);
    if (analyseRes) {
      const codeExtensions = new Set(['.wxml', '.wxss', '.wxs', '.js', '.json', '.json', '.ts']);
      const { files } = analyseRes;
      const filesToDelete = files
        .filter((f) => {
          if (codeExtensions.has(f.ext) && !f.moduleId && f.path?.startsWith?.('materials/')) {
            return true;
          }
          return false;
        })
        .map((f) => f.path);

      return Promise.all(
        filesToDelete.map(async (f) => {
          await fs.remove(path.join(this.mpCIProject.projectPath, f));
          return f;
        }),
      );
    } else {
      throw new Error('小程序代码静态分析失败');
    }
  }

  /**
   * 小程序-预览
   */
  async _ciPreview(options = {}): Promise<IMiniProgramBuildOutput> {
    // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
    // this.api.runtime.isLocal() && this.api.console.pause();
    !process.env.CLOUDBASE_CIID && pauseConsoleOutput();

    const previewOptions = {
      version: '0.0.1',
      desc: 'CloudBase Framework 一键预览',
      pagePath: 'pages/index/index',
      searchQuery: '',
      scene: 1011,
      qrcodeFormat: 'image',
      qrcodeOutputPath: './qrcode.jpg',
      ...(this.resolvedInputs.miniprogramOptions?.previewOptions || {}),
      ...options,
    };

    const qrcodeOutputDest = path.resolve(this.resolvedInputs.outputPath, previewOptions.qrcodeOutputPath);

    const result = await previewMp({
      project: this.mpCIProject,
      ...(previewOptions as any),
      qrcodeOutputDest,
    }).catch((err) => {
      return err;
    });
    // this.api.runtime.isLocal() && this.api.console.resume();
    !process.env.CLOUDBASE_CIID && resumeConsoleOutput();

    if (result?.subPackageInfo) {
      return {
        link: qrcodeOutputDest,
      };
    } else {
      throw new Error(`小程序（预览版）部署失败 ${result}`);
    }
  }

  /**
   * 小程序-上传
   */
  async _ciUpload(options = {}): Promise<IMiniProgramBuildOutput> {
    // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
    // this.api.runtime.isLocal() && this.api.console.pause();
    !process.env.CLOUDBASE_CIID && pauseConsoleOutput();

    const uploadOptions = {
      version: '1.0.0',
      desc: 'CloudBase Framework 一键上传',
      ...(this.resolvedInputs.miniprogramOptions.uploadOptions || {}),
      ...(options || {}),
    };

    const result = await uploadMp({
      project: this.mpCIProject,
      ...uploadOptions,
    }).catch((err) => {
      return err;
    });
    // this.api.runtime.isLocal() && this.api.console.resume();
    !process.env.CLOUDBASE_CIID && resumeConsoleOutput();

    if (result?.subPackageInfo) {
      return {
        version: uploadOptions.version,
      };
    } else {
      throw new Error(`小程序（体验版）部署失败 ${result}`);
    }
  }
}

export const deployer = LowcodeDeployer;

const originalStdoutWrite = process.stdout.write.bind(process.stdout);
const originalStderrWrite = process.stderr.write.bind(process.stderr);
let previousStdoutWrite = process.stdout.write.bind(process.stdout);
let previousStderrWrite = process.stderr.write.bind(process.stderr);
// 暂停控制台输出
function pauseConsoleOutput() {
  previousStdoutWrite = process.stdout.write.bind(process.stdout);
  process.stdout.write = () => {
    return true;
  };
  previousStderrWrite = process.stderr.write.bind(process.stderr);
  process.stderr.write = () => {
    return true;
  };
}
// 恢复控制台输出
function resumeConsoleOutput(original = false) {
  if (original) {
    process.stdout.write = originalStdoutWrite;
    process.stderr.write = originalStderrWrite;
  } else {
    process.stdout.write = previousStdoutWrite;
    process.stderr.write = previousStderrWrite;
  }
}

function zipDir(src, dist) {
  return new Promise((resolve, reject) => {
    // create a file to stream archive data to.
    let output = fs.createWriteStream(dist);
    let archive = archiver('zip', {
      zlib: { level: 9 }, // Sets the compression level.
    });
    output.on('close', resolve);
    archive.on('error', reject);
    archive.directory(src, false);
    archive.pipe(output);
    archive.finalize();
  });
}
